censre3 <- function(id,C,Q,X,Z,alpha,D,sigma,eps=.001,nsize=500,burnin=25,
                   silent=F,maxloops=100,iseed){
# Purpose: Implements Monte Carlo EM estimator for mixed effects model with 
#          left and/or right censored data. Assumes normal errors and 
#          normal distributions for random effects.
# 
# Required:
#  id = unique identifier for each individual/unit
#
#  C = censoring indicator; C = -1 means left censored, C = 0 means
#      uncensored, C = 1 means right censored
#
#  Q = response variable; if C = 0, Q is the uncensored observation; if
#      C != 0, Q is the censoring level
#
#  X = design matrix for fixed effects; generate using model.matrix or
#      by hand
#
#  Z = design matrix for random effects; generate using model.matrix or  
#      by hand
#
#  alpha = initial values for fixed effects (optional)
#
#  D = initial guess for the covariance matrix of the random effects
#
#  S = initial guess for the within-subject residual variance
#
# Optional:
#  eps = max. relative change in parameters between successive iterates
#        to achieve convergence; typically this will be larger than one
#        might use in a deterministic algorithm
#  
#  nsize = starting number of Monte Carlo samples to compute expected 
#          sufficient statistics on each person during E-step; this
#          number is doubled if the absolute change in any parameter is
#          less than the estimated Monte Carlo standard error for that
#          parameter
#
#  burnin = number of Monte Carlo samples to discard during Gibbs 
#           sampling; the sampler typically burns in quite rapidly
#           in this application
#
#  silent = if T, do not display intermediate results
#
#  maxloops = maximun number of EM loops
#
#  iseed = random number seed (a negative integer)
#
# Reference: Hughes JP. Mixed effects models with censored data with 
#            application to HIV RNA levels. Biometrics 55:625-629, 1999.
#
  nsubj <- length(unique(id))
  n <- rle(id)$lengths
  p <- ncol(X)
  q <- ncol(Z)
  sumn <- sum(n)
  if (missing(alpha)) alpha <- solve(t(X)%*%X)%*%t(X)%*%Q
  info <- matrix(0,p,p)
  storage.mode(X) <- "double"
  storage.mode(Z) <- "double"
  storage.mode(D) <- "double"
  storage.mode(info) <- "double"
  set.seed(iseed)
  zzz <- .Fortran("censre3",
                  as.integer(burnin),
                  as.integer(nsize),
                  as.integer(nsubj),
                  as.integer(sumn),
                  as.integer(n),
                  as.integer(p),
                  as.integer(q),
                  X,
                  Z,
                  as.double(Q),
                  as.integer(C),
                  alpha=as.double(alpha),
                  D=D,
                  sigma=as.double(sigma),
                  info=info,
                  as.integer(silent),
                  as.double(eps),
                  as.integer(maxloops))

   var.alpha <- solve(zzz$info)
  list(alpha=zzz$alpha,D=zzz$D,sigma=zzz$sigma,var=var.alpha)
  }
